import Dispatcher from "./dispatcher";

type RedirectInterceptorOpts = { maxRedirections?: number }

export declare function createRedirectInterceptor (opts: RedirectInterceptorOpts): Dispatcher.DispatchInterceptor
